﻿using System.Web;
using Sitecore.Publishing;
using Sitecore.Security.Accounts;
using Sitecore.Web;

namespace Skillcore.PersonalizationPreview
{
    public class PersonalizationPreviewContext
    {
        public bool Allowed { get; set; }
        public bool Enabled { get; set; }

        public static PersonalizationPreviewContext Current
        {
            get
            {
                PersonalizationPreviewContext context = null;
                HttpContext httpContext = HttpContext.Current;
                if (httpContext != null)
                {
                    context = httpContext.Items["PersonalizationPreviewContext"] as PersonalizationPreviewContext;
                    if (context == null)
                    {
                        context = new PersonalizationPreviewContext();
                        httpContext.Items["PersonalizationPreviewContext"] = context;
                    }
                }
                return context;
            }
        }

        private PersonalizationPreviewContext()
        {
            if (Sitecore.Context.PageMode.IsPreview || WebUtil.GetQueryString("mode") == "preview")
            {
                PreviewManager.RestoreUser();
                PreviewManager.SetUserContext();
                User user = Sitecore.Context.User;
                if (user.IsAuthenticated)
                {
                    Allowed = true;
                    if (user.Profile["PersonalizationPreviewEnabled"] == "true")
                    {
                        Enabled = true;
                    }
                }
            }
        }

        public void Toggle()
        {
            User user = Sitecore.Context.User;
            if (user.IsAuthenticated)
            {
                if (user.Profile["PersonalizationPreviewEnabled"] != "true")
                {
                    user.Profile["PersonalizationPreviewEnabled"] = "true";
                }
                else
                {
                    user.Profile["PersonalizationPreviewEnabled"] = "false";
                }
                user.Profile.Save();
            }
        }
    }
}