﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using Sitecore.Data.Items;
using Sitecore.Mvc.Extensions;
using Sitecore.Mvc.Pipelines.Response.GetXmlBasedLayoutDefinition;
using Sitecore.Mvc.Presentation;
using Sitecore.Text;

namespace Skillcore.PersonalizationPreview.Pipelines.MvcGetXmlBasedLayoutDefinition
{
    public class DuplicatePersonalizedRenderings : GetXmlBasedLayoutDefinitionProcessor
    {
        private const string RuleInfoTemplate = "&personalization-preview-variant-id={0}&personalization-preview-variant-name={1}";

        public override void Process(GetXmlBasedLayoutDefinitionArgs args)
        {
            if (!Sitecore.Context.PageMode.IsPreview || !PersonalizationPreviewContext.Current.Enabled)
                return;

            if (args.Result == null)
                return;

            Item item = PageContext.Current.Item;
            if (item == null)
                return;

            foreach (XElement device in args.Result.Elements(XName.Get("d")))
            {
                List<XElement> newRenderings = new List<XElement>();
                List<XElement> orignalRenderings = device.Elements(XName.Get("r")).ToList();

                foreach (XElement rendering in orignalRenderings)
                {
                    bool replaced = false;

                    XElement rls = rendering.Elements(XName.Get("rls")).FirstOrDefault();
                    XElement ruleset;
                    if (rls != null && (ruleset = rls.Elements(XName.Get("ruleset")).FirstOrDefault()) != null)
                    {
                        XElement defaultRule = ruleset.Descendants(XName.Get("rule")).FirstOrDefault(r => r.GetAttributeValueOrEmpty("uid") == "{00000000-0000-0000-0000-000000000000}");
                        if (defaultRule != null)
                        {
                            XElement defaultConditions = defaultRule.Elements(XName.Get("conditions")).FirstOrDefault();

                            if (defaultConditions != null)
                            {
                                string renderingParameters = rendering.GetAttributeValueOrEmpty("par");
                                if (!String.IsNullOrEmpty(renderingParameters))
                                {
                                    renderingParameters += "&";
                                }
                                renderingParameters += "personalization-preview=true";
                                
                                foreach (XElement rule in ruleset.Elements(XName.Get("rule")))
                                {
                                    XElement renderingCopy = new XElement(rendering);
                                    string ruleInfo = String.Format(RuleInfoTemplate, rule.GetAttributeValueOrEmpty("uid"), new UrlString(rule.GetAttributeValueOrEmpty("name")));
                                    renderingCopy.SetAttributeValue(XName.Get("par"), renderingParameters + ruleInfo); 
                                    List<XElement> copiedRules = renderingCopy.Descendants(XName.Get("rule")).ToList();
                                    foreach (XElement copiedRule in copiedRules)
                                    {
                                        if (copiedRule.GetAttributeValueOrEmpty("uid") != rule.GetAttributeValueOrEmpty("uid"))
                                        {
                                            copiedRule.Remove();
                                        }
                                    }
                                    XElement newConditions = new XElement(defaultConditions);

                                    XElement conditions = renderingCopy.Descendants(XName.Get("conditions")).FirstOrDefault();
                                    if (conditions != null)
                                    {
                                        conditions.ReplaceWith(newConditions);
                                    }

                                    newRenderings.Add(renderingCopy);
                                    replaced = true;
                                }
                            }
                        }
                    }
                    if (!replaced)
                    {
                        newRenderings.Add(rendering);
                    }
                }

                if (orignalRenderings.Count != newRenderings.Count)
                {
                    device.RemoveNodes();
                    device.Add(newRenderings.Cast<object>().ToArray());
                }
            }
        }
    }
}
