﻿using System.IO;
using Sitecore.Mvc.Pipelines.Response.RenderRendering;
using Sitecore.Mvc.Presentation;

namespace Skillcore.PersonalizationPreview.Pipelines.MvcRenderRendering
{
    public class CreateCommentTags : RenderRenderingProcessor
    {
        private const string PersonalizationPreviewCommentStartTag = "<!--personalization-preview-start::{0}::{1}::{2}::{3}-->";
        private const string PersonalizationPreviewCommentEndTag = "<!--personalization-preview-end::{0}-->";

        public string Type { get; set; }

        public override void Process(RenderRenderingArgs args)
        {
            if (Sitecore.Context.PageMode.IsPreview && PersonalizationPreviewContext.Current.Enabled)
            {
                bool personalizationPreview = args.Rendering.Parameters["personalization-preview"] == "true";
                if (personalizationPreview)
                {
                    if (Type == "start")
                    {
                        WritePersonalizationInfoStart(args.Writer, args);
                    }
                    if (Type == "end")
                    {
                        WritePersonalizationInfoEnd(args.Writer, args);
                    }
                }
            }
        }

        private void WritePersonalizationInfoStart(TextWriter writer, RenderRenderingArgs args)
        {
            writer.Write(PersonalizationPreviewCommentStartTag,
                args.Rendering.UniqueId,
                args.Rendering.Parameters["personalization-preview-variant-id"],
                args.Rendering.RenderingItem.DisplayName,
                args.Rendering.Parameters["personalization-preview-variant-name"]);
        }

        private void WritePersonalizationInfoEnd(TextWriter writer, RenderRenderingArgs args)
        {
            writer.Write(PersonalizationPreviewCommentEndTag, args.Rendering.UniqueId);
        }
    }
}