var personalizationPreviewModule = function() {
	var sections = [];
	var selector, selectorHeader, galleryList;
	var ribbonDocument, previewCommandButton;
	var lastRow = null;

	var getCommentNodes = function() {
		function traverseDom(curr_element) {
			var comments = new Array();
			if (curr_element.nodeName == "#comment" || curr_element.nodeType == 8) {
				comments[comments.length] = curr_element;
			}
			else if(curr_element.childNodes.length>0) {
				for (var i = curr_element.childNodes.length - 1; i >= 0; i--) {
					comments = comments.concat(traverseDom(curr_element.childNodes[i]));
				}
			}
			return comments;
		}
	   return traverseDom(document.getElementsByTagName("html")[0]);
	}

	var addLoadEvent = function(func) {
		var oldonload = window.onload;
		if (typeof window.onload != 'function') {
			window.onload = func;
		} else {
			window.onload = function() {
				if (oldonload) {
					oldonload();
				}
				func();
			}
		}
	}

	var findVariantElements = function(startComment, sectionId) {
		var elements = [];
		var sibling = startComment.nextSibling;
		while (!sibling.nodeValue || sibling.nodeValue != 'personalization-preview-end::' + sectionId) {
			elements.push(sibling);
			sibling = sibling.nextSibling;
		}
		return elements;
	}

	var getKeys = function (array) {
		var keys = [];
		for (var key in array) {
			if (key === 'length' || !array.hasOwnProperty(key)) continue;
			keys.push(key);
		}
		return keys;
	}

	var personalizedSection = function(sectionId, sectionName) {
		var self = this;
		self.id = sectionId;
		self.name = sectionName;
		self.variants = [];
	}

	var sectionVariant = function(variantId, variantName, variantStart, variantElements) {
		var self = this;
		self.id = variantId;
		self.name = variantName;
		self.holder = document.createElement("DIV");
		self.startNode = variantStart;
		self.elements = variantElements;
	}

	var initialize = function () {
	    var nodes = getCommentNodes();
	    for (var i = nodes.length - 1; i >= 0; i--) {
	        var node = nodes[i];
	        if (node.nodeValue && node.nodeValue.indexOf('personalization-preview-start:') == 0) {
	            var parts = node.nodeValue.split('::');
	            var sectionId = parts[1];
	            var variantId = parts[2];
	            var sectionName = parts[3];
	            var variantName = parts[4];
	            var variant = new sectionVariant(variantId, variantName, node, findVariantElements(node, sectionId));

	            var section = sections[sectionId];

	            if (section == null) {
	                section = new personalizedSection(sectionId, sectionName);
	                sections[sectionId] = section;
	            }
	            section.variants.push(variant);
	        }
	    }

	    var galleryDiv = ribbonDocument.createElement('DIV');
	    galleryDiv.setAttribute('class', 'scRibbonGallery');
        galleryDiv.setAttribute('style', 'overflow-y: scroll; height: 42px;');
	    galleryList = ribbonDocument.createElement('DIV');
	    galleryDiv.appendChild(galleryList);
	    var keys = getKeys(sections);
	    if (keys.length) {
	        galleryList.setAttribute('class', 'scRibbonGalleryList');
	        galleryList.setAttribute('style', 'width: auto; height: auto; min-width: 200px;');
	        createSelector();

	        for (var i = keys.length - 1; i >= 0; i--) {
	            var key = keys[i];
	            var section = sections[key];
	            createVariantSelector(section, i + 1);
	            for (var j = section.variants.length - 2; j >= 0; j--) {
	                hideVariant(section.variants[j]);
	            }
	        }
	    } else {
	        galleryList.setAttribute('class', 'scRibbonToolbarText');
	        galleryList.innerHTML = 'There are no personalized components on the current item';
	    }
	    previewCommandButton.parentNode.insertBefore(galleryDiv, previewCommandButton.nextSibling);
	};

	var personalizePreview = function () {
	    var ribbon = document.getElementById('scWebEditRibbon');
	    if (ribbon) {
	        ribbonDocument = ribbon.contentWindow.document;
	        previewCommandButton = ribbonDocument.getElementById('PersonalizationPreviewButton');
	        if ((previewCommandButton.getAttribute('class') + '').indexOf('scRibbonToolbarLargeButtonDown') >= 0) {
	            setTimeout(initialize, 1);
	        }
	    }
	}
	
	var createSelector = function() {
		selector = document.createElement('TABLE');
		galleryList.appendChild(selector);
	};

	var hideVariant = function(variant) {
		for (var k = 0; k < variant.elements.length; k++) {
			variant.holder.appendChild(variant.elements[k]);
		}
	}
	
	var selectVariant = function() {
		var option = this.options[this.selectedIndex],
		    variantId = option.value,
		    sectionId = option.getAttribute('data-section-id'),
			i;
		
		var section = sections[sectionId];
		if (section.id == sectionId) {
			var selectedVariant;
			for (i = section.variants.length - 1; i >= 0; i--) {
				var variant = section.variants[i];
				if (variant.id == variantId) {
					selectedVariant = variant;
				} else {
					hideVariant(variant);
				}
			}

			for (i = selectedVariant.elements.length - 1; i >= 0; i--) {
				selectedVariant.startNode.parentNode.insertBefore(selectedVariant.elements[i], selectedVariant.startNode.nextSibling);
			}
		}
	};

	var createVariantSelector = function(section, index) {
		var tr = document.createElement('TR');
		var td = document.createElement('TD');
		var label = document.createElement('LABEL');
		td.appendChild(label);
		tr.appendChild(td);
		td = document.createElement('TD');
		tr.appendChild(td);
		label.setAttribute('for', 'personalization-preview-selector-' + index);
		label.innerHTML = section.name;
		var select = document.createElement('SELECT');
		select.onchange = selectVariant;
		select.setAttribute('id', 'personalization-preview-selector-' + index);
		select.setAttribute('style', 'font-size: 1em; width: 100%');
		for (var i = section.variants.length - 1; i >= 0; i--) {
			var variant = section.variants[i];
			var option = document.createElement('OPTION');
			option.setAttribute('data-section-id', section.id);
			option.setAttribute('value', variant.id);
			option.innerHTML = variant.name;
			if (i == section.variants.length - 1) {
				option.setAttribute('selected', 'selected');
			}
			select.appendChild(option);
		}
		td.appendChild(select);
		selector.insertBefore(tr, lastRow);
		lastRow = tr;
	}

	addLoadEvent(personalizePreview);
}();